import React, {Component} from 'react'
import PropTypes from 'prop-types'
import { is, fromJS } from 'immutable'
import { Collapse, notification } from 'antd'
import moment from 'moment'

import Api from '@/api'
import UtilsDM from '@/utils/utils-datamanage.js'
import asyncComponent from '@/utils/asyncComponent'
import MKEmitter from '@/utils/events.js'
import TimerTask from '@/utils/timer-task.js'
import './index.scss'

// 通用组件
const MainSearch = asyncComponent(() => import('@/tabviews/zshare/topSearch'))
const MainAction = asyncComponent(() => import('@/tabviews/zshare/actionList'))
const MainTable = asyncComponent(() => import('@/tabviews/custom/components/share/normalTable'))
const NormalHeader = asyncComponent(() => import('@/tabviews/custom/components/share/normalheader'))

const { Panel } = Collapse

class NormalTable extends Component {
  static propTpyes = {
    config: PropTypes.object
  }

  state = {
    BID: '',              // 上级ID
    BData: '',            // 上级组件行数据
    config: {},           // 页面配置信息，包括按钮、搜索、显示列、标签等
    actions: null,        // 按钮集
    columns: null,        // 显示列
    setting: null,        // 页面全局设置：数据源、按钮及显示列固定、主键等
    data: [],             // 列表数据集
    selectedData: [],     // 已选表格数据
    total: 0,             // 总数
    loading: false,       // 列表数据加载中
    pageIndex: 1,         // 页码
    pageSize: 10,         // 每页数据条数
    orderBy: '',          // 排序
    search: '',           // 搜索条件数组，使用时需分场景处理
    statFValue: [],       // 合计值
    allSearch: null
  }

  loaded = false
  supModules = []

  /**
   * @description 初始化处理
   * 1、 initdata 为打印时使用的数据集
   */
  UNSAFE_componentWillMount () {
    const { config } = this.props

    let _config = fromJS(config).toJS()
    let _data = null
    let BID = ''
    let BData = ''

    if (_config.setting.supModule) {
      BData = window.GLOB.CacheData.get(_config.setting.supModule)
    } else {
      BData = window.GLOB.CacheData.get(_config.$pageId)
    }
    if (BData) {
      BID = BData.$BID || ''
    }

    if (_config.wrap.controlField) {
      if (_config.wrap.controlVal) {
        _config.wrap.controlVal = _config.wrap.controlVal.split(',')
      } else {
        _config.wrap.controlVal = ['']
      }
    }

    let setting = {..._config.setting, ..._config.wrap}

    if (setting.selected !== 'always' && setting.selected !== 'init' && setting.selected !== 'sign') {
      setting.selected = 'false'
    } else {
      setting.orisel = true
    }

    if (_config.setting.sync === 'true') {
      _config.setting.onload = 'false'

      if (window.GLOB.SyncData.has(_config.dataName)) {
        _data = window.GLOB.SyncData.get(_config.dataName) || []
  
        if (_config.$cache) {
          Api.writeCacheConfig(_config.uuid, fromJS(_data).toJS(), BID)
        }
  
        _config.setting.sync = 'false'
  
        _data = _data.map((item, index) => {
          item.key = index
          item.$$uuid = item[_config.setting.primaryKey] || ''
          item.$$key = '' + item.key + item.$$uuid
          item.$$BID = BID || ''
          item.$$BData = BData || ''
          item.$Index = index + 1 + ''
  
          if (_config.absFields) {
            _config.absFields.forEach(f => {
              if (!isNaN(item[f])) {
                item[f] = Math.abs(item[f])
              }
            })
          }
  
          if (setting.controlField) {
            if (setting.controlVal.includes(item[setting.controlField] + '')) {
              item.$disabled = true
            }
          }
  
          return item
        })
  
        if (setting.selected !== 'false' && _data.length > 0) {
          setTimeout(() => {
            MKEmitter.emit('mkCheckTopLine', _config.uuid, '', setting.selected)
          }, 200)
          if (setting.selected === 'init') {
            setting.selected = 'false'
          }
        }
  
        this.loaded = true
  
        window.GLOB.SyncData.delete(_config.dataName)
      }
    }
    
    if (_config.wrap.collapse === 'true') {
      _config.wrap.title = _config.wrap.title || ' '
    }

    _config.style = _config.style || {}

    let allSearch = null
    if (_config.colsCtrls && _config.colsCtrls.length > 0) {
      allSearch = [{key: 'BID', value: BID || ''}]
      allSearch.push(..._config.$searches)
      if (_config.setting.useMSearch) {
        let mainSearch = window.GLOB.SearchBox.get(_config.$searchId) || []
        allSearch.push(...mainSearch)
      }
    } else {
      _config.colsCtrls = null
    }

    let columns = _config.cols
    if (_config.hasExtend) {
      columns = this.getCols(_config.cols, BData, _config.setting.extendTime)
    }

    this.setState({
      pageSize: setting.pageSize || 10,
      BID: BID || '',
      BData: BData || '',
      title: _config.wrap.title,
      data: _data,
      config: _config,
      setting: setting,
      actions: _config.action,
      columns: columns,
      search: _config.$searches,
      allSearch
    })
  }

  shouldComponentUpdate (nextProps, nextState) {
    return !is(fromJS(this.props), fromJS(nextProps)) || !is(fromJS(this.state), fromJS(nextState))
  }

  componentDidMount () {
    const { config } = this.state

    MKEmitter.addListener('reloadData', this.reloadData)
    MKEmitter.addListener('refreshLineData', this.refreshLineData)
    MKEmitter.addListener('resetSelectLine', this.resetParentParam)
    MKEmitter.addListener('queryModuleParam', this.queryModuleParam)
    MKEmitter.addListener('refreshByButtonResult', this.refreshByButtonResult)

    if (config.setting.useMSearch) {
      MKEmitter.addListener('searchRefresh', this.searchRefresh)
    }

    if (config.setting.sync === 'true') {
      MKEmitter.addListener('transferSyncData', this.transferSyncData)
    }

    if (config.timer) {
      this.timer = new TimerTask()
      this.timer.init(config.uuid, config.timer, config.timerRepeats, () => {
        this.setState({
          pageIndex: 1
        }, () => {
          this.loadmaindata(true, 'true', '', 'timer')
        })
      })
    }

    this.initExec()

    this.autoExec()
  }

  /**
   * @description 组件销毁，清除state更新，清除快捷键设置
   */
  componentWillUnmount () {
    this.setState = () => {
      return
    }
    MKEmitter.removeListener('reloadData', this.reloadData)
    MKEmitter.removeListener('searchRefresh', this.searchRefresh)
    MKEmitter.removeListener('refreshLineData', this.refreshLineData)
    MKEmitter.removeListener('resetSelectLine', this.resetParentParam)
    MKEmitter.removeListener('transferSyncData', this.transferSyncData)
    MKEmitter.removeListener('queryModuleParam', this.queryModuleParam)
    MKEmitter.removeListener('refreshByButtonResult', this.refreshByButtonResult)

    this.timer && this.timer.stop()
  }

  initExec = () => {
    const { config, setting, BID } = this.state

    if (config.$cache) {
      if (config.$time && !setting.laypage) {
        if (!this.loaded) {
          Api.getLCacheConfig(config.uuid, config.$time, BID).then(res => {
            if (!res.valid && config.setting.onload === 'true') {
              setTimeout(() => {
                this.loadmaindata(false, 'true', '', 'init')
              }, config.setting.delay || 0)
            }
  
            if (!res.data || this.loaded) return

            this.setState({data: res.data.map((item, index) => {
              item.key = index
              item.$$uuid = item[config.setting.primaryKey] || ''
              item.$$key = '' + item.key + item.$$uuid
              item.$Index = index + 1 + ''
    
              if (config.absFields) {
                config.absFields.forEach(f => {
                  if (!isNaN(item[f])) {
                    item[f] = Math.abs(item[f])
                  }
                })
              }
    
              if (setting.controlField) {
                if (setting.controlVal.includes(item[setting.controlField] + '')) {
                  item.$disabled = true
                }
              }
              
              return item
            })})
          })
        }
      } else {
        if (!this.loaded) {
          Api.getLCacheConfig(config.uuid, 0, BID).then(res => {
            if (!res.data || this.loaded) return
    
            this.setState({data: res.data.map((item, index) => {
              item.key = index
              item.$$uuid = item[config.setting.primaryKey] || ''
              item.$$key = '' + item.key + item.$$uuid
              item.$Index = index + 1 + ''
    
              if (config.absFields) {
                config.absFields.forEach(f => {
                  if (!isNaN(item[f])) {
                    item[f] = Math.abs(item[f])
                  }
                })
              }
    
              if (setting.controlField) {
                if (setting.controlVal.includes(item[setting.controlField] + '')) {
                  item.$disabled = true
                }
              }
              
              return item
            })})
          })
        }

        if (config.setting.onload === 'true') {
          setTimeout(() => {
            this.loadmaindata(false, 'true', '', 'init')
          }, config.setting.delay || 0)
        }
      }
    } else if (config.setting.onload === 'true') {
      setTimeout(() => {
        this.loadmaindata()
      }, config.setting.delay || 0)
    }
  }

  autoExec = (times) => {
    const { config } = this.state

    if (!config.wrap.autoExec) return

    let btn = document.getElementById('button' + config.wrap.autoExec)

    this.autoTimer && clearTimeout(this.autoTimer)

    if (btn) {
      MKEmitter.emit('triggerBtnId', config.wrap.autoExec, [])
    } else if (!times || times < 20) {
      times = times ? times + 1 : 1
      this.autoTimer = setTimeout(() => {
        this.autoExec(times)
      }, 1000)
    }
  }

  transferSyncData = (syncId) => {
    const { config } = this.state

    if (config.$syncId !== syncId) return

    const { setting, BID, BData } = this.state

    let _data = window.GLOB.SyncData.get(config.dataName) || []

    if (config.$cache) {
      Api.writeCacheConfig(config.uuid, fromJS(_data).toJS(), BID)
    }

    _data = _data.map((item, index) => {
      item.key = index
      item.$$uuid = item[config.setting.primaryKey] || ''
      item.$$key = '' + item.key + item.$$uuid
      item.$$BID = BID || ''
      item.$$BData = BData || ''
      item.$Index = index + 1 + ''

      if (config.absFields) {
        config.absFields.forEach(f => {
          if (!isNaN(item[f])) {
            item[f] = Math.abs(item[f])
          }
        })
      }

      if (setting.controlField) {
        if (setting.controlVal.includes(item[setting.controlField] + '')) {
          item.$disabled = true
        }
      }
      
      return item
    })

    if (setting.selected !== 'false' && _data.length > 0) {
      setTimeout(() => {
        MKEmitter.emit('mkCheckTopLine', config.uuid, '', setting.selected)
      }, 200)
      if (setting.selected === 'init') {
        this.setState({setting: {...setting, selected: 'false'}})
      }
    }

    this.loaded = true

    this.setState({data: _data})

    window.GLOB.SyncData.delete(config.dataName)

    MKEmitter.removeListener('transferSyncData', this.transferSyncData)
  }

  getCols = (cols, BData, time) => {
    let _cols = []
    let weeks = ['星期日', '星期一', '星期二', '星期三', '星期四', '星期五', '星期六']

    let excols = (col) => {
      let _s = []
      let _marks = col.marks || []
      let names = []
      let fields = []

      for (let i = 0; i < col.quota; i++) {
        fields.push(col.field + (i === 0 ? '' : i))

        if (col.colUnit === 'day') {
          let hasweek = col.dayFormat.indexOf('week') > -1
          let format = col.dayFormat.replace(/\sweek/, '')
          let day = moment().add(col.shift + i, 'days').format(format)

          if (hasweek) {
            let _w = moment().add(col.shift + i, 'days').weekday()
            day = day + ' ' + weeks[_w]
          }

          names.push(day)
        } else {
          let format = col.hourFormat
          if (format === 'H point') {
            format = 'H点'
          }

          let hour = moment(time).add(col.shift + i, 'hours').format(format)

          if (format === 'h:00') {
            if (hour === '12:00') {
              hour = hour + ' pm'
            } else if (hour === moment(time).add(col.shift + i, 'hours').format('H:00')) {
              hour = hour + ' am'
            } else {
              hour = hour + ' pm'
            }
          }
          names.push(hour)
        }
      }

      if (col.supField) {
        names = []
        let val = ''
        if (BData) {
          let field = col.supField.toLowerCase()
          Object.keys(BData).forEach(key => {
            if (key.toLowerCase() === field) {
              val = BData[key] + ''
            }
          })
        }

        if (val) {
          names = val.split(',')

          if (names.length > fields.length) {
            names.length = fields.length
          } else if (fields.length > names.length) {
            fields.length = names.length
          }
        }
      }

      if (names.length === 0) return _s

      _s = fields.map((f, i) => {
        return {
          type: 'text',
          Align: col.Align,
          Width: col.Width,
          field: f,
          label: names[i],
          uuid: col.uuid + i,
          marks: _marks.filter(mark => mark.field[0] === f || !fields.includes(mark.field[0]))
        }
      })

      return _s
    }

    cols.forEach(col => {
      if (col.type === 'extend') {
        let exs = excols(col)
        _cols.push(...exs)
      } else if (col.type === 'colspan') {
        let subs = []
        col.subcols.forEach(subcol => {
          if (subcol.type === 'extend') {
            let exs = excols(subcol)
            subs.push(...exs)
          } else {
            subs.push(col)
          }
        })

        if (subs.length) {
          _cols.push({...col, subcols: subs})
        }
      } else {
        _cols.push(col)
      }
    })

    return _cols
  }

  resetCols = (BData) => {
    const { config } = this.state

    let columns = this.getCols(config.cols, BData, config.setting.extendTime)

    this.setState({
      columns: columns,
      data: [],
      selectedData: [],
      total: 0
    })
  }

  /**
   * @description 主表数据加载
   * @param { Boolean } reset  表格是否重置
   * @param { String }  repage 表格是否重置页码
   */
  async loadmaindata (reset, repage, id, type) {
    const { setting, config, search, orderBy, BID, pageIndex, pageSize, BData } = this.state

    if (setting.supModule && !BID && setting.supKey !== 'false') { // BID 不存在时，不做查询
      this.setState({
        data: [],
        selectedData: [],
        loading: false,
        total: 0
      })
      
      MKEmitter.emit('resetSelectLine', config.uuid, '', '') // 广播数据切换
      reset && MKEmitter.emit('resetTable', config.uuid, 'true') // 列表重置
      if (setting.$hasSyncModule) {
        MKEmitter.emit('syncBalconyData', config.uuid, [], false)
      }
      if (config.$hasTopModule) {
        window.GLOB.CacheData.set(config.uuid + 'tb', { $$empty: true, $$uuid: '' })
        MKEmitter.emit('mkPublicData', config.uuid + 'tb', { $$empty: true, $$uuid: '' })
      }

      this.loaded = true
      this.requestId = ''
      return
    }

    let searches = fromJS(search).toJS()
    if (config.setting.useMSearch) { // 主表搜索条件
      let mainSearch = window.GLOB.SearchBox.get(config.$searchId) || []
      let keys = config.$s_keys || []
      mainSearch.forEach(item => {
        if (keys.includes(item.key.toLowerCase())) return

        searches.push(item)
      })
    }

    if (config.$s_req && searches.filter(item => item.required && item.value === '').length > 0) {
      return
    }

    if (type !== 'timer') {
      this.setState({
        loading: true
      })
    }

    let _orderBy = orderBy || setting.order
    let param = UtilsDM.getQueryDataParams(setting, searches, _orderBy, pageIndex, pageSize, BID)

    this.requestId = config.uuid + new Date().getTime()

    let result = await Api.genericInterface(param, '', '', this.requestId)
    if (result.status) {
      if (result.$requestId && this.requestId !== result.$requestId) return
      
      this.loaded = true
      if (config.$cache && type === 'init') {
        Api.writeCacheConfig(config.uuid, result.data || [], BID)
      }

      if (repage === 'false' && result.data && result.data.length === 0 && result.total > 0 && pageIndex > 1) {
        let _pageIndex = Math.ceil(result.total / pageSize)

        if (_pageIndex < pageIndex) {
          MKEmitter.emit('resetTable', config.uuid, 'repage', _pageIndex)
          this.setState({
            pageIndex: _pageIndex,
            data: [],
            selectedData: [],
            total: result.total
          }, () => {
            this.loadmaindata()
          })
          return
        }
      }
      
      if ((setting.selected !== 'false' || (setting.orisel && id)) && result.data && result.data.length > 0) {
        setTimeout(() => {
          MKEmitter.emit('mkCheckTopLine', config.uuid, id, setting.selected)
        }, 200)
        if (setting.selected === 'init') {
          this.setState({setting: {...setting, selected: 'false'}})
        }
      } else {
        MKEmitter.emit('resetSelectLine', config.uuid, '', '') // 广播数据切换
        if (setting.$hasSyncModule) {
          MKEmitter.emit('syncBalconyData', config.uuid, [], false)
        }
      }
      
      reset && MKEmitter.emit('resetTable', config.uuid, repage) // 列表重置

      let start = 1
      if (setting.laypage) {
        start = pageSize * (pageIndex - 1) + 1
      }

      let data = result.data.map((item, index) => {
        item.key = index
        item.$$uuid = item[setting.primaryKey] || ''
        item.$$key = '' + item.key + item.$$uuid
        item.$$BID = BID || ''
        item.$$BData = BData || ''
        item.$Index = start + index + ''

        if (config.absFields) {
          config.absFields.forEach(f => {
            if (!isNaN(item[f])) {
              item[f] = Math.abs(item[f])
            }
          })
        }

        if (setting.controlField) {
          if (setting.controlVal.includes(item[setting.controlField] + '')) {
            item.$disabled = true
          }
        }
        
        return item
      })

      let total = result.total || 0
      if (config.setting.custompage && data.length) {
        total = data[data.length - 1].mk_total || 0
      }

      let allSearch = null
      if (config.colsCtrls) {
        allSearch = [{key: 'BID', value: BID || ''}]
        allSearch.push(...searches)
      }

      this.setState({
        data: data,
        selectedData: [],
        allSearch: allSearch,
        total: total,
        loading: false
      })

      if (config.$hasTopModule) {
        window.GLOB.CacheData.set(config.uuid + 'tb', data[0] || { $$empty: true, $$uuid: '' })
        MKEmitter.emit('mkPublicData', config.uuid + 'tb', data[0] || { $$empty: true, $$uuid: '' })
      }
      
      if (config.timer && config.clearField && result.data && result.data[0]) {
        let vals = (config.clearValue || '').split(',')
        if (vals.includes(result.data[0][config.clearField])) {
          this.timer && this.timer.stop()
        }
      }

      UtilsDM.querySuccess(result)
    } else {
      this.setState({
        loading: false
      })
      this.timer && this.timer.stop()
      
      UtilsDM.queryFail(result)
    }
  }

  /**
   * @description 获取单行数据
   */ 
  async loadLinedata (id, position) {
    const { setting, config, search, orderBy, BID, pageIndex, pageSize, BData } = this.state

    if (config.forbidLine) {
      this.reloadtable()
      return
    }

    let searches = fromJS(search).toJS()
    if (config.setting.useMSearch) { // 主表搜索条件
      let mainSearch = window.GLOB.SearchBox.get(config.$searchId) || []
      let keys = config.$s_keys || []
      mainSearch.forEach(item => {
        if (keys.includes(item.key.toLowerCase())) return

        searches.push(item)
      })
    }

    this.setState({
      loading: true
    })

    let _orderBy = orderBy || setting.order
    let param = UtilsDM.getQueryDataParams(setting, searches, _orderBy, pageIndex, pageSize, BID, id)

    let result = await Api.genericInterface(param)
    if (result.status) {
      if (position === 'line_grid' && (!result.data || !result.data[0])) {
        this.loadmaindata(true, 'false')
        return
      }
      
      let data = fromJS(this.state.data).toJS()
      let selectedData = fromJS(this.state.selectedData).toJS()

      if (result.data && result.data[0]) {
        let _data = result.data[0]
        _data.$$uuid = _data[setting.primaryKey] || ''
        _data.$$BID = BID || ''
        _data.$$BData = BData || ''

        if (setting.controlField) {
          if (setting.controlVal.includes(_data[setting.controlField] + '')) {
            _data.$disabled = true
          }
        }
  
        if (config.absFields) {
          config.absFields.forEach(f => {
            if (!isNaN(_data[f])) {
              _data[f] = Math.abs(_data[f])
            }
          })
        }
  
        data = data.map(item => {
          if (item.$$uuid === _data.$$uuid) {
            _data.key = item.key
            _data.$$key = '' + item.key + item.$$uuid
            _data.$Index = item.$Index
            return _data
          } else {
            return item
          }
        })

        if (!_data.$Index) {
          this.setState({
            loading: false
          })
          return
        }

        selectedData = selectedData.map(item => {
          if (_data.$$uuid === item.$$uuid) {
            return _data
          }
          return item
        })

        if (selectedData.length && _data.$$uuid === selectedData[selectedData.length - 1].$$uuid) {
          MKEmitter.emit('resetSelectLine', config.uuid, _data.$$uuid || '', _data)
        }
  
        this.setState({
          data,
          selectedData,
          loading: false
        })

        if (config.setting.$hasSyncModule) {
          MKEmitter.emit('syncBalconyData', config.uuid, selectedData, data.length === selectedData.length)
        }
      } else {
        let index = data.findIndex(item => item.$$uuid === id)

        if (index === -1) {
          this.setState({
            loading: false
          })
          return
        }

        data = data.filter(item => item.$$uuid !== id)
        data = data.map((item, i) => {
          item.key = i
          item.$$key = '' + item.key + item.$$uuid

          return item
        })

        this.setState({
          data: data,
          loading: false
        })

        setTimeout(() => {
          MKEmitter.emit('resetTable', config.uuid, 'delete', index)
        }, 20)
      }
    } else {
      this.setState({
        loading: false
      })
      
      UtilsDM.queryFail(result)
    }
  }

  /**
   * @description 搜索条件改变时，重置表格数据
   * 含有初始不加载的页面，修改设置
   */
  refreshbysearch = (searches) => {
    const { setting, BID } = this.state

    if (setting.supModule && !BID && setting.supKey !== 'false') {
      notification.warning({
        top: 92,
        message: window.GLOB.dict['sup_key_req'] || '需要上级主键值！',
        duration: 5
      })
      this.setState({
        search: searches
      })
      return
    }

    this.setState({
      pageIndex: 1,
      search: searches
    }, () => {
      this.loadmaindata(true, 'true')
    })
  }

  /**
   * @description 表格条件改变时重置数据（分页或排序）
   */
  refreshbytable = (pagination, filters, sorter) => {
    if (sorter.order) {
      let _chg = {
        ascend: 'asc',
        descend: 'desc'
      }
      sorter.order = _chg[sorter.order]
    }

    this.setState({
      pageIndex: pagination.current,
      pageSize: pagination.pageSize,
      orderBy: (sorter.field && sorter.order) ? `${sorter.field} ${sorter.order}` : ''
    }, () => {
      this.loadmaindata()
    })
  }

  /**
   * @description 表格刷新
   */
  reloadtable = (btn, id) => {
    if (!btn || btn.resetPageIndex !== 'false') {
      this.setState({
        pageIndex: 1
      }, () => {
        this.loadmaindata(true, 'true', id)
      })
    } else {
      this.loadmaindata(true, 'false', id)
    }
  }

  /**
   * @description 导出Excel时，获取页面搜索排序等参数
   */
  queryModuleParam = (menuId, callback) => {
    const { config, orderBy, search, setting} = this.state

    if (config.uuid !== menuId) return

    let searches = search ? fromJS(search).toJS() : []
    if (config.setting.useMSearch) { // 主表搜索条件
      let mainSearch = window.GLOB.SearchBox.get(config.$searchId) || []
      let keys = config.$s_keys || []
      mainSearch.forEach(item => {
        if (keys.includes(item.key.toLowerCase())) return

        searches.push(item)
      })
    }

    callback({
      orderBy: orderBy || setting.order,
      search: searches
    })
  }

  reloadData = (menuId, id) => {
    const { config } = this.state

    if (config.uuid !== menuId) return

    if (!id) {
      this.reloadtable()
    } else {
      this.loadLinedata(id)
    }
  }

  resetParentParam = (MenuID, id, data) => {
    const { config, setting } = this.state

    if (config.supNodes) {
      if (!config.supNodes.includes(MenuID)) return
      this.supModules = this.supModules.filter(item => item.key !== MenuID)
      let bid = ''
      let _data = null

      if (id) {
        this.supModules.push({key: MenuID, value: id, data})
      }
      if (this.supModules.length > 0) {
        bid = this.supModules[this.supModules.length - 1].value
        _data = this.supModules[this.supModules.length - 1].data
      }

      if (bid !== this.state.BID || bid !== '') {
        this.setState({ BID: bid, BData: _data, pageIndex: 1 }, () => {
          this.loadmaindata(true, 'true')
        })
      }
    } else {
      if (!setting.supModule || setting.supModule !== MenuID) return
  
      if (id !== this.state.BID || id !== '') {
        if (config.parCtrl) {
          this.resetCols(data)
        }
        this.setState({
          pageIndex: 1,
          BID: id,
          BData: data
        }, () => {
          if (!setting.checkBid) {
            this.loadmaindata(true, 'true')
          }
        })
      }
    }
  }

  /**
   * @description 按钮执行完成后页面刷新
   * @param {*} menuId     // 菜单Id
   * @param {*} position   // 刷新位置
   * @param {*} btn        // 执行的按钮
   */
  refreshByButtonResult = (menuId, position, btn, id, lines) => {
    const { config, BID } = this.state

    if (config.uuid !== menuId) return

    if (config.supNodes) {
      if (['mainline', 'maingrid', 'popclose'].includes(position)) {
        let supNode = this.supModules[this.supModules.length - 1]
        config.supNodes.forEach((item, i) => {
          setTimeout(() => {
            if (supNode && supNode.key === item) {
              MKEmitter.emit('reloadData', item, supNode.value)
            } else {
              MKEmitter.emit('reloadData', item)
            }
          }, i * 10)
        })
      } else {
        if (position === 'line' || position === 'line_grid') {
          if (lines && lines.length === 1 && !config.forbidLine) {
            this.loadLinedata(lines[0].$$uuid, position)
          } else {
            this.reloadtable(btn, id)
          }
        } else {
          this.reloadtable(btn, id)
        }
      }
    } else {
      if (position === 'line' || position === 'line_grid') {
        if (lines && lines.length === 1 && !config.forbidLine) {
          this.loadLinedata(lines[0].$$uuid, position)
        } else {
          this.reloadtable(btn, id)
        }
      } else if (['mainline', 'maingrid', 'popclose'].includes(position) && config.setting.supModule) {
        MKEmitter.emit('reloadData', config.setting.supModule, position === 'maingrid' ? '' : BID)
      } else {
        this.reloadtable(btn, id)
      }
    }
  }

  refreshLineData = (menuId, btn, uuid, count) => {
    const { config } = this.state

    if (config.uuid !== menuId) return
    
    let _data = fromJS(this.state.data).toJS().map(item => {
      if (item.$$uuid === uuid) {
        item[btn.field] = count
      }
      return item
    })
    let _selectedData = fromJS(this.state.selectedData).toJS().map(item => {
      if (item.$$uuid === uuid) {
        item[btn.field] = count
      }
      return item
    })

    this.setState({
      data: _data,
      selectedData: _selectedData
    })
  }

  searchRefresh = (searchId) => {
    const { config } = this.state

    if (config.$searchId !== searchId) return
    
    this.reloadtable()
  }

  render() {
    const { BID, setting, actions, config, columns, selectedData, BData, data, allSearch } = this.state

    let style = {...config.style}
    if (config.wrap.empty === 'hidden' && (!data || data.length === 0)) {
      style.display = 'none'
    }

    let content = <>
      {config.search.length ?
        <MainSearch BID={BID} config={config} refreshdata={this.refreshbysearch}/> : null
      }
      {actions.length > 0 ? <MainAction
        BID={BID}
        setting={setting}
        actions={actions}
        BData={BData}
        columns={config.columns}
        selectedData={selectedData}
      /> : (setting.tableType || config.search.length > 0 ? <div className="mk-action-space" style={{height: '25px'}}></div> : null)}
      <MainTable
        setting={setting}
        columns={columns}
        MenuID={config.uuid}
        allSearch={allSearch}
        data={this.state.data}
        fields={config.columns}
        total={this.state.total}
        parCtrl={config.parCtrl}
        colsCtrls={config.colsCtrls}
        lineMarks={config.lineMarks}
        loading={this.state.loading}
        refreshdata={this.refreshbytable}
        statFValue={this.state.statFValue}
        chgSelectData={(selects) => this.setState({selectedData: selects})}
      />
    </>

    return (
      <div className="custom-normal-table" id={'anchor' + config.uuid} style={style}>
        {config.wrap.collapse === 'true' ? <Collapse bordered={false} defaultActiveKey="1" expandIconPosition="right">
          <Panel forceRender={true} header={<NormalHeader config={config}/>} key="1">
            {content}
          </Panel>
        </Collapse> : <>
          <NormalHeader config={config}/>
          {content}
        </>}
      </div>
    )
  }
}

export default NormalTable